#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU-Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;
use diagnostics;

# -----------------------------------------------------------------------------
package UHU::detect;

use UHU::common;

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
_capacity
disks
detectVideoDriver
getDDC
);

# ----- General helper subs ---------------------------------------------------
sub _model($) {
	my ($device) = @_;

	chomp_(`getmodel $device`) or "Ismeretlen";
}

sub _capacity($) {
	my ($device) = @_;

	((chomp_(`blockdev --getsize $device`) or 0) *
		(chomp_(`blockdev --getss $device`) or 512));
}

sub _realpath($) {
	my ($device) = @_;

	chomp_(`realpath $device`) or "";
}

sub disks() {
	my $size;
	(map {
		{
			model      => _model "$_/disc",
			capacity   => _capacity "$_/disc",
			device     => _realpath "$_/disc",
			partitions => [
				map {
					{
						size   => _capacity $_,
						device => _realpath $_,
					}
				} (< $_/part* >)
			]
		}
	} (< /dev/discs/disc* >)),
	{
			model      => "RAID",
			capacity   => undef,
			device     => undef,
			partitions => [
				map {
					($size = (`blockdev --getsize $_` * 512)) ? {
						size   => $size,
						device => $_,
					} : ()
				} (< /dev/md/* >)
			]
	}
}

sub getVideoDriver($) {
	my %CardsDB;
	my ($detectedname) = @_;

	my $currentname;
	foreach (cat '/usr/share/ldetect-lst/Cards+') {
		next unless /^NAME\s|^DRIVER\s|^NOCLOCKPROBE|^SEE\s/;
		$currentname = $1 if /^NAME\s+(\S.+\S)\s*$/;
		next unless $currentname;
		if (/^DRIVER\s+(\S+)\s*$/) {
			$CardsDB{$currentname}{driver} = $1;
		}
		$CardsDB{$currentname}{noclockprobe} = 1 if /^NOCLOCKPROBE\s*$/;
		%{$CardsDB{$currentname}} = %{$CardsDB{$1}} if /^SEE\s+(\S+)\s*$/;
	}

	$CardsDB{$detectedname}{driver} or 'vesa';
}

sub detectVideoDriver() {
	my $name = cat1 '/etc/sysconfig/card';
	(getVideoDriver($name), $name);
}

sub getDDC() {
	open DDC, '/usr/sbin/ddcinfo |';
	<DDC>;    # ^
	<DDC>;    # | - videokatya infok
	<DDC>;    # v
	(chomp_ <DDC>, chomp_ <DDC>, chomp_ <DDC>, chomp_ <DDC>);
}

# ----- Beatiful perl :) ------------------------------------------------------
"Mit mond a sundiszno a surolokefen?";
