#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU-Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;
use diagnostics;

# -----------------------------------------------------------------------------
package UHU::gtk;

use Gtk;
use Gtk::Gdk::ImlibImage;
use Gtk::GladeXML;

use UHU::common;

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
set_combo_list set_combo_hash get_combo_key
insert_node
w
plug load_ui load_ui_into_container load_ui_into_plug
load_image setup_images image
set_cursor set_pointer_cursor set_wait_cursor
set_if_not
Xres
popup
dir_select
gtk_iterate
msg
);

# -----------------------------------------------------------------------------
my %combodata;
my %images;
my $images;

sub set_combo_list($$$) {
	my ($g, $combo, $list) = @_;

	debug "set_combo_list";
	debug "-> $_" foreach (@$list);

	w($g, $combo)->set_popdown_strings(@$list);
}

sub set_combo_hash($$$) {
	my ($g, $combo, $hash) = @_;

	debug "set_combo_hash";

	$combodata{$g}{$combo} = $hash;

	set_combo_list $g, $combo, [sort keys %$hash];

	#	set_combo_list $g, $combo, [sort(map {$hash->{$_}->{text}} keys(%$hash))];
}

sub get_combo_key($$) {
	my ($g, $combo) = @_;

	$combodata{$g}{$combo}{ w($g, $combo)->entry->get_text };
}

sub insert_node($$$;$$) {
	my ($gtktree, $parent, $text, $is_leaf, $icon) = @_;

	my @icon = ();
	if ($icon) {
		@icon = load_image($icon);
	} else {
		@icon = (undef, undef);
	}
	if ($gtktree) {
		$gtktree->insert_node(
			(
				$parent
				or undef
			),
			undef,
			(
			$text
			or ""
			),
			5,
			@icon,
			undef,
			undef,
			$is_leaf ? 1 : 0,
			0
		);
	}
}

sub plug($) {
	my ($wid) = @_;

	die "Nincs hova csatlakozni!" unless $wid;

	my $plug = new Gtk::Plug $wid;
	$plug->signal_connect('destroy',      sub {exit});
	$plug->signal_connect('delete_event', sub {exit});

	$plug->show;

	$plug;
}

sub w($$) {
	my ($root, $widget_name) = @_;

	$root->get_widget($widget_name);
}

sub load_ui($) {
	my ($uifile) = @_;

	my $g = new Gtk::GladeXML($uifile);
	$g->signal_autoconnect_from_package('main');

	$g;
}

sub load_ui_into_container($$$) {
	my ($uifile, $sourcename, $target) = @_;

	my $g = load_ui($uifile);

	my $source = w($g, $sourcename);

	$source->unrealize;
	$source->hide;
	$source->reparent($target);
	$source->show;
	$source->realize;

	$g;
}

sub load_ui_into_plug($$$) {
	my ($uifile, $sourcename, $XID) = @_;

	load_ui_into_container $uifile, $sourcename, plug($XID);
}

sub load_image($) {
	my ($name) = @_;

	debug "loading image: [$name]";

	my $image = Gtk::Gdk::ImlibImage->load_image($name)
	or return (undef, undef);
	$image->render($image->rgb_width, $image->rgb_height);

	($image->move_image, $image->move_mask);
}

sub setup_images($) {
	my ($hash) = @_;

	$images = $hash;
}

sub image($) {
	my ($id) = @_;

	$images{$id} = [load_image($images->{$id})] unless $images{$id};

	@{ $images{$id} };
}

sub set_cursor($;$) {
	my ($widget, $cursor) = @_;

	my %table = (
		pointer => 68,
		wait    => 150
	);

	$cursor = 'pointer' unless $cursor;

	$cursor = $table{$cursor} if $table{$cursor};

	$widget->window->set_cursor(Gtk::Gdk::Cursor->new($cursor));
}

sub set_if_not($$) { 
#beallitja az entry tartalmat, ha nem az
# hogy ne kuldjon megint signalt
	my ($entry,$text)=@_;
	$text='' unless $text;
	$entry->set_text($text)
		if ($entry->get_text ne $text);	
}
sub Xres {
	(Gtk::Gdk->screen_width, Gtk::Gdk->screen_height);
}

sub popup(%) {
	my %prop = @_;

	# TODO:
	# - bgcolor support

	my $popup = new Gtk::Window 'toplevel';
	$popup->set_position('center');
	$popup->set_modal(1);
	$popup->signal_connect('delete_event', sub {1});

	my $frame1 = new Gtk::Frame;
	$frame1->set_shadow_type('out');
	$popup->add($frame1);

	my $frame2 = new Gtk::Frame;
	$frame2->set_shadow_type('in');
	$frame2->set_border_width(3);
	$frame1->add($frame2);

	my $vbox = new Gtk::VBox(0, 10);
	$vbox->set_border_width(10);
	$frame2->add($vbox);

	my $label = new Gtk::Label;
	$label->set_text($prop{msg});
	$vbox->pack_start_defaults($label);

	$vbox->add(new Gtk::HSeparator);

	my $hbuttonbox = new Gtk::HButtonBox;
	$vbox->pack_start($hbuttonbox, 0, 0, 0);

	$prop{buttons} = ['OK', undef] unless $prop{buttons};
	while (@{ $prop{buttons} }) {
		my $label   = shift @{ $prop{buttons} };
		my $handler = shift @{ $prop{buttons} };
		my @argv = ();
		if (ref($handler) eq 'ARRAY') {
			($handler, @argv) = @$handler;
		}

		my $button = new Gtk::Button;
		$button->label($label);
		$button->signal_connect(
			'clicked',
			sub {
				$popup->hide;
				$popup->destroy;
				Gtk->main_iteration while Gtk->events_pending;
				&$handler(@argv) if $handler;
			}
		);
		$hbuttonbox->add($button);
		$button->grab_focus if ($prop{default} or $label) eq $label;
	}

	$popup->show_all;
	set_cursor $popup;
}

sub msg($) {
	my ($msg) = @_;

	popup(msg => $msg);
}

sub warning($) {
	my ($msg) = @_;

	popup(msg => $msg, bgcolor => "#FF0000");
}

sub dir_select($$) {
	my ($title, $callback) = @_;

	my $selector = new Gtk::FileSelection($title);
	$selector->set_modal(1);
	$selector->hide_fileop_buttons;

	#	$selector->file_list->set_sensitive(0);
	#	$selector->file_list->signal_handlers_destroy;
	$selector->file_list->parent->hide;

	#	$selector->selection_entry->set_sensitive(0);
	#	$selector->selection_entry->hide;
	$selector->selection_text->hide;
	$selector->ok_button->signal_connect(
		'clicked',
		sub {
			my $dir = $selector->get_filename;
			if (-d $dir) {
				&$callback($dir);
				$selector->destroy;
			} else {
				popup(msg => 'Nincs ilyen knyvtr!');
			}
		}
	);
	$selector->cancel_button->signal_connect(
		'clicked',
		sub {
			$selector->destroy;
		}
	);
	$selector->show;
}

sub gtk_iterate() {
	Gtk->main_iteration while Gtk->events_pending;
}

# ----- Initialization --------------------------------------------------------
init Gtk;
init Gtk::Gdk::ImlibImage;

# ----- Beatiful perl :) ------------------------------------------------------
"Megy a csiga hazafele.";
